#include "macro-lib.cc"

struct Icon {
  const char * name;
  bool disappears;
  // Look for 3 horizontal RGB values (range 0-255) at given offset
  Point pixelOffset;
  int pixels [3][3];

  Point getTopLeft(int position) {
    // Position: 1 at top-right, increasing leftward
    return (Point) { tale.width - 64 * position, 0 };
  }

  bool isAt(int position) {
    pixelOffset.x = pixelOffset.x % 64;
    Point p = getTopLeft(position) + pixelOffset;
    //cout << "Trying " << name << " in pos " << position << ":" << endl;
    for (int i = 0; i < 3; i++) {
      int * rgb = &pixels[i][0];
      double match[4] = { rgb[0] / 255.0, rgb[1] / 255.0, rgb[2] / 255.0, 0.01 };
      Point test = p + (Point) { i, 0 };
      Color c = getColor(tale.w, test);
      //cout << "  " << test << " - " << c;
      if (!c.match(match)) {
	//cout << " != ";
      } else {
	//cout << " == ";
      }
      //cout << "(" << match[0] << ", " << match[1] << ", " << match[2] << ") +/- " << match[3] << endl;
      if (!c.match(match))  return false;
    }
    return true;
  }

  int getPosition() {
    // Return position number of icon on screen, or 0 if not found
    for (int i = 1; i <= 9; i++) {
      if (isAt(i))  return i;
    }
    return 0;
  }

  void clickPos(int position) {
    clickWait(getTopLeft(position));
  }
};

Icon slateIcon = { "slate", true, (Point) { 996, 24 },
		   { {237,237,234}, {109,109,107}, {170,170,164} } };
Icon grassIcon = { "grass", true, (Point) { 1113, 32 },
		   { {216,189,181}, {47,52,62}, {41,155,25} } };
Icon clayIcon = { "clay", true, (Point) { 1106, 74 },
		  { {127,59,61}, {181,155,148}, {162,152,140} } };
/*
Disabled since timers changed a lot in T4, and I haven't bothered to
get these to work properly.

Icon dirtIcon = { "dirt", false, (Point) { 1118, 40 },
		  { {88,72,58}, {243,204,161}, {128,106,80} } };
Icon limestoneIcon = { "limestone", false, (Point) { 0, 0 },
		       { {255,0,0}, {0,255,0}, {0,0,255} } };
Icon fishIcon = { "fish", false, (Point) { 981, 10 },
		  { {227,195,79}, {135,122,98}, {174,175,183} } };
Icon dowseIcon = { "dowse", true, (Point) { 1177, 15 },
		   { {255,248,192}, {223,169,62}, {85,48,0} } };
*/

Icon iconList[] = {
  slateIcon,
  grassIcon,
  clayIcon,
  /*
  dirtIcon,
  limestoneIcon,
  fishIcon,
  dowseIcon
  */
};
const int numIcons = sizeof(iconList) / sizeof(Icon);


int main(int argc, char *argv[]) {
  int checkIcons[argc];
  int hitCount[argc];
  int numToCheck = 0;

  for (int argIndex = 1; argIndex < argc; argIndex++) {
    for (int i = 0; i < numIcons; i++) {
      if (strcmp(argv[argIndex], iconList[i].name) == 0) {
	checkIcons[numToCheck] = i;
	hitCount[numToCheck] = 0;
	numToCheck++;
      }
    }
  }

  if (numToCheck == 0) {
    cout << argv[0] << " [ ";
    for (int i = 0; i < numIcons; i++) {
      if (i != 0) {
	cout << " | ";
      }
      cout << iconList[i].name;
    }
    cout << " ] " << endl;
    exit(1);
  }
  
  initMacro();

  bool finished = 0;
  while (!finished) {
    for (int i = 0; i < numToCheck; i++) {
      Icon * icon = &iconList[checkIcons[i]];
      int pos = icon->getPosition();
      if (pos > 0) {
	icon->clickPos(pos);
	hitCount[i]++;
	cout << "Click " << hitCount[i] << " on " << icon->name << endl;
      }
    }
    waitForRefresh();
  }

  endMacro();
}
